@@ -5,6 +5,15 @@
 # usethis::use_vignette('interactive-plotting')
 
 ## NOTE -- make sure the the OS API key is set in memory before compiling.
+osdatahub::set_os_key(key)
+# delete the created *.Rmd files
+# optionally, rename the .Rmd.orig files and edit, then proceed
+# file.rename("vignettes/ngd-api.Rmd.orig", "vignettes/ngd-api.Rmd")
+# file.rename("vignettes/extents.Rmd.orig", "vignettes/extents.Rmd")
+# file.rename("vignettes/plotting-results.Rmd.orig", "vignettes/plotting-results.Rmd")
+# file.rename("vignettes/interactive-plotting.Rmd.orig", "vignettes/interactive-plotting.Rmd")
+# file.rename("vignettes/crs-pitfalls.Rmd.orig", "vignettes/crs-pitfalls.Rmd")
+#
 
 # quick renaming of files
 
@@ -12,6 +21,7 @@ file.rename("vignettes/ngd-api.Rmd", "vignettes/ngd-api.Rmd.orig")
 file.rename("vignettes/extents.Rmd", "vignettes/extents.Rmd.orig")
 file.rename("vignettes/plotting-results.Rmd", "vignettes/plotting-results.Rmd.orig")
 file.rename("vignettes/interactive-plotting.Rmd", "vignettes/interactive-plotting.Rmd.orig")
+file.rename("vignettes/crs-pitfalls.Rmd", "vignettes/crs-pitfalls.Rmd.orig")
 
 # set API key
 
@@ -21,6 +31,7 @@ knit("vignettes/ngd-api.Rmd.orig", "vignettes/ngd-api.Rmd")
 knit("vignettes/extents.Rmd.orig", "vignettes/extents.Rmd")
 knit("vignettes/plotting-results.Rmd.orig", "vignettes/plotting-results.Rmd")
 knit("vignettes/interactive-plotting.Rmd.orig", "vignettes/interactive-plotting.Rmd")
+knit("vignettes/crs-pitfalls.Rmd.orig", "vignettes/crs-pitfalls.Rmd")
 # knit("vignettes/json-paging.Rmd.orig", "vignettes/json-paging.Rmd")
 
 # Move the figures to the vignettes folder.
